        ; **************************************************************
        ; *** 2D-Stepping Procedure - For Intel 386  (c) R.Heuberger ***
        ; **************************************************************
        ; 
        ;       Eingangs-Bedingungen:  X-Schritte: EAX
        ;                              Y-Schritte: EBX
        ;                Vorschub-Geschwindigkeit: EDX
        ;
        ; Defenitions - local buffer ( offsets ) :  
        ;  di[0]  = PHYSICAL-xstepp      di[4]  = PHYSICAL-ystepp 
        ;  di[8]  = Backup edx           di[12] = Temp. & Main-Speed
        ;  di[16] = Backup al            di[20] = Backup al
        ;  di[24] = Step_M               di[28] = step_1
        ;  di[32] = Stepword, step_M     di[36] = Stepword, step_1
        ;  di[40] = err_1                di[44] = kerr_1
        ;  di[48] = gerr_1               di[52] = Save eax
        ;  di[56] = Save ebx
        ;
        ;
        TITLE kreis
        .Model Large
        .386
        .code
        ;
kreis   proc
        push    di              ; Save di
        push    si              ; Save si
        push    sp              ; Save sp
        ;
        ;mov     eax,4000d       ; $- DEBUG purpose, xstepp% -$
        ;mov     ebx,2000d       ; $- DEBUG purpose, ystepp% -$
        ;mov     edx,1600d       ; $- DEBUG purpose, speed%  -$
        ;
        ;and     eax,eax         ; IF   xstepp value not equal "0"
        ;jne     sbegin          ; THEN start program.
        ;and     ebx,ebx         ; IF   ystepp value not equal "0"
        ;je      abo             ; THEN exit
        and      edx,edx
        je       abo
        ;
sbegin: lea     di, OFFSET lobuf   ; di = Pointer to local buffer.
        mov     cs:[di]+56,ebx     ; Save ebx-Register
        mov     cs:[di]+8,edx      ; Save speed value local ( 32 bit ).
        ;
        add     cs:[di],eax        ;-; PHYSICAL: Calculate current X-Position.
        add     cs:[di]+4,ebx      ;-; PHYSICAL: Calculate current Y-Position.
        ;
        and     eax,eax         ; Test, = Find X-direction.
        js      Xtopos          ;  X-DIR = backwards.
        mov     cs:[di]+52,eax  ; SAVE eax-Register !
        mov     al,00001000b    ; Set X - forward bit
        jmp     Chey            ; 
Xtopos: neg     eax             ; Make X-Step posetive !
        mov     cs:[di]+52,eax  ; SAVE eax-Register !
        mov     al,00000000b    ; Clear - al, = X Backwards.
        ;
Chey:   and     ebx,ebx         ; Test, = Find Y-diriction.
        js      Ytopos          ; Y-DIR  already done.
        or      al,00010000b    ; SET Y - forward bit.
        jmp     Sdir
Ytopos: neg     ebx             ; Make Y - Step posetive !
        mov     cs:[di]+56,ebx  ; SAVE ebx-Register AGAIN!
        ;
Sdir:   mov     cs:[di]+16,al   ; Save al - Register
        mov     cs:[di]+20,al   ; Save al - Register 
        mov     dx,378H         ; Base-Address
        out     dx,al           ; Set CSR-Output-register = Axis directions.
        inc     dx              ; POINT to CSR - controll-register
        inc     dx              ;   "    "    "        "
        mov     al,1            ; al <--"1"
        out     dx,al           ; Strobe = HIGH
        dec     al              ; All <--"0"
        out     dx,al           ; Strobe = LOW
        ; 
        ;   - Find axis_M, = axis with greatest movement.
        ;   - Check & order deltas, axis_M, axis_1 
        ;
        mov     eax,cs:[di]+52       ; RESTORE eax - register  
        cmp     ebx,eax              ; Compare Xstepp with Ystepp.
        jg      YgtX                 ; Ystepp > Xstepp
        mov     cs:[di]+24,eax       ;-> X-axis has greatest movement! 
        mov     cs:[di]+28,ebx       ;\
        mov     al,00000001b         ;>>
        mov     cs:[di]+32,al        ;>>>> Setup sequenze if X-axis
        mov     al,00000010b         ;>>        is Main-Axis.
        mov     cs:[di]+36,al        ;/
        jmp     Cabre                ;
YgtX:   mov     cs:[di]+24,ebx       ;-> Y-axis has greatest movement!
        mov     cs:[di]+28,eax       ;\
        mov     al,00000010b         ;>>
        mov     cs:[di]+32,al        ;>>>> Setup sequenze if Y-axis
        mov     al,00000001b         ;>>      is Main-Axis.
        mov     cs:[di]+36,al        ;/
        ;
        ;
        ;
        ;  ***  Calculate values for Bresenham's Algorithm. ***
        ;        Note: eax-register already saved.. index 0
        ;
Cabre:  mov     eax,cs:[di]+24          ; Get delta_M      =>eax
        mov     ebx,cs:[di]+28          ; Get delta_1      =>ebx
        mov     ecx,cs:[di]+24          ; Backup  delta_M  =>ecx
        sal     ebx,1                   ; delta_1 = delta_1 * 2
        sal     eax,1                   ; delta_M = delta_M * 2
        mov     cs:[di]+44,ebx          ; SAVE "kerr_1",( prepare,if err_1>0 )
        mov     cs:[di]+12,ebx          ; Backup delta_1 => "temp1" 
        sub     ebx,ecx                 ; err_1 = 2*delta_1-delta_M
        mov     cs:[di]+40,ebx          ; SAVE err_1
        mov     ebx,cs:[di]+12          ; RECALL delta_1 temp1 =>ebx
        sub     ebx,eax                 ; "gerr_1" = 2*delta_1 - 2*delta_M
        mov     cs:[di]+48,ebx          ; SAVE "gerr_1"
        ;
        ;
        ;===============================;
Mlpre:  mov     ebx,cs:[di]+8           ; Get requested speed value and save
        mov     cs:[di]+12,ebx          ;  it at current speed value.
        mov     ebx,cs:[di]+24          ; Recall main-loop count.
Mloop:  mov     dx,379H                 ;+++>  CSR-Address
        in      al,dx                   ;+++>  Get Printerport-Status
        TEST    al,80H                  ;+++>  Test BUSY - bit
        jz      ausga                   ;+++>  External STOP !
        call    sistepp                 ; Ein 2D-Schritt
        dec     ebx                     ; Main-
        jnz     Mloop                   ;      Loop...
        mov     ebx,cs:[di]+84          ; Get Flag : 
        and     ebx,ebx                 ;       If NOT trapezodial mode
        jz      abo                     ;         Than exit !
        ;===============================;
        ;
        ;
abo:    pop     sp                      ; Restore sp - register
        pop     si                      ; Restore si - register
        pop     di                      ; Restore di - register
        ret
        ;
ausga:  mov     cs:[di]+60,DWORD PTR 1 ; SET Emerg. Abort-Flag
        jmp     abo                     ;  
        ;
kreis   endp
        ;
        ;
        ; Stepping - Subroutine. Use Bresenham's Algorith to
        ; execute single or multiple axis movement for one stepp.
        ;
sistepp proc
        mov     al,cs:[di]+16    ; RESTORE al-register
        or      al,cs:[di]+32    ; ENABLE exis_m
        ;
        mov     edx,cs:[di]+40   ; Recall "err_1" =>edx
        and     edx,edx          ; Test edx,=err_1
        jg      Egt_1            ; Branch if "err-1" > 0
        mov     edx,cs:[di]+44   ; Recall "kerr_1"
        add     cs:[di]+40,edx   ; "err_1" = "err_1" + "kerr_1"
        jmp     Dostep           ; DISABLE axis_1 & jump
Egt_1:  or      al,cs:[di]+36    ; ENABLE  axis_1
        mov     edx,cs:[di]+48   ; Recall "gerr_1"
        add     cs:[di]+40,edx   ; "err_1" = "err_1" + "gerr_1"
        ;
Dostep: mov     cs:[di]+20,al    ; SAVE al - Register
        ; -----------------------
        mov     dx,378H         ; Base-Address
        mov     al,cs:[di]+16   ; DATA - LOW level
        out     dx,al           ; Set ALL =Output=0
        inc     dx              ; POINT to controll-register
        inc     dx              ;   "    "    "        "
        mov     al,1            ; AL <--"1"
        out     dx,al           ; Strobe = HIGH
        dec     al              ; All <--"0"
        out     dx,al           ; Strobe = LOW 
        dec     dx              ; Point to Base
        dec     dx              ;   "    "   "
        mov     al,cs:[di]+20   ; set DATA - HIGH level
        out     dx,al           ; Set ALL =Output=1
        inc     dx              ; POINT to controll-register
        inc     dx              ;  "     "    "        "
        mov     al,1            ; AL <--"1"
        out     dx,al           ; Strobe = HIGH
        dec     al              ; All <--"0"
        out     dx,al           ; Strobe = LOW
        ; ----------------------- 
        ; Delay:
        mov     edx,cs:[di]+12
nl:     nop
        dec     edx
        jnz     nl
        ;
        ret                     ; Return from Single-Step.
        ;
        ;gab     db   2 dup(3)
lobuf   db  56 dup(0)
endb1   dd   1 dup(12345678h)
        ;
sistepp endp
        ;
        ;
        end
